-----------------------------------------
--             FIR Filter              -- 
-- corr_filter.vhd                     --
-- Other files:                        --
--         corr_filter_pkg.vhd         --
--         corr_filter_userio.vhd      --
--                                     --
-- For capstone, Wulfsberg, 2009/2010  --
--                                     --
-- Author: Bryan Woolsey               --
-- Email: ragnarkon@nau.edu            --
-- VHDL Version:  VHDL-87,             --
--					 updated to VHDL-93--
-- Date: 10-17-09                      --
-- Last Update: 4-22-10                --
-----------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;  -- use numeric_std for VHDL-93+
use work.corr_filter_pkg.all;
use IEEE.std_logic_unsigned.all;

entity corr_filter is
	generic (
		head_size : positive := work.corr_filter_pkg.header_size;
		out_bits : positive := 11
	);
	port (
		clk, rst : in std_logic;  -- clock & reset
		Yn_out : out std_logic_vector(out_bits-1 downto 0);  -- output
		Xn_in : in std_logic_vector(3 downto 0);  -- input
		channel_sel : in headervaltype -- input from user i/o, new as of 2-23-10
	);
end corr_filter;

architecture arch of corr_filter is
	
	constant h : positive := head_size - 1;
    shared variable header_value:	headervaltype := work.corr_filter_pkg.ch_null;  --Change array values to change header
	
	type REGTYPE is array (h downto 0) of std_logic_vector(3 downto 0);
	type MULTTYPE is array (h downto 0) of std_logic_vector(out_bits-1 downto 0);

	signal REG : REGTYPE;
	signal MULT : MULTTYPE;
	
--	alias conv_lvector is IEEE.std_logic_arith.conv_std_logic_vector;    -- Can't use in VHDL-87
begin
	
    genreg : for j in h-1 downto 0 generate
	
        master : process (rst, clk)
        begin
        
            if (rst='1') then
				REG(j) <= (REG(j)'range => '0');
				MULT(j) <= (MULT(j)'range => '0');
			elsif (clk'event and clk ='1') then
				REG(j) <= REG(j+1);
				MULT(j) <= std_logic_vector((to_signed(header_value(j), 7)) * signed(REG(j)));
			end if;
			
        end process;
		
    end generate;
	
	output_topreg : process (clk, rst)
	variable SUM : std_logic_vector(out_bits-1 downto 0);
	begin
			if (rst = '1') then
				REG(h) <= (REG(h)'range => '0');
				MULT(h) <= (MULT(h)'range => '0');
				SUM := (SUM'range => '0');
			elsif (clk'event and clk='1') then
				REG(h) <= Xn_in;
				MULT(h) <= std_logic_vector((to_signed(header_value(h), 7)) * signed(REG(h)));
				if (SUM /= 0) then
					SUM := (SUM'range => '0');
				end if;
				for n in h downto 0 loop
					SUM := std_logic_vector(signed(SUM) + signed(MULT(n)));
				end loop;
				Yn_out <= SUM;  --output
			end if;
	end process;
	  
	change_header : process(channel_sel)
	begin
		header_value := channel_sel;
	end process;
end arch;