-----------------------------------------
--          FIR Filter TestBench       -- 
-- corr_filter_tb.vhd                  --
--                                     --
-- For capstone, Wulfsberg, 2009/2010  --
--                                     --
-- Author: Bryan Woolsey               --
-- Email: ragnarkon@nau.edu            --
-- VHDL Version:  VHDL-93              --
-- Date: 4-22-10                       --
-- Last Update: 4-22-10                --
-----------------------------------------

library ieee;
use ieee.std_logic_1164.all; 

library work;
use work.corr_filter_pkg.all;

entity corr_filter_tb is 
	port
	(
		clock : in std_logic;
		reset : in std_logic;
		user_chan_sel : in std_logic_vector(3 downto 0);
		filter_in : in std_logic_vector(3 downto 0);
		filter_out : out std_logic_vector(10 downto 0);
		hex1_out : out std_logic_vector(6 downto 0);
		hex2_out : out std_logic_vector(6 downto 0);
		error_hex : out std_logic
	);
end corr_filter_tb;

architecture mappinz of corr_filter_tb is 

	component corr_filter_userio
		port(channel_sel : in std_logic_vector(3 downto 0);
			channel_sel_io : out std_logic_vector(3 downto 0);
			channel_sel_out : out headervaltype
		);
	end component;
	
	component corr_filter
		generic (head_size : positive; out_bits : positive);
	
		port(clk, rst : in std_logic;
			Yn_out : out std_logic_vector(out_bits-1 downto 0);
			Xn_in : in std_logic_vector(3 downto 0);
			channel_sel : in headervaltype
		);
	end component;
	
	component hex_counter
		port(bits : in std_logic_vector(3 downto 0);
			bits2 : in std_logic_vector(3 downto 0);
			sevenseg : out std_logic_vector(6 downto 0);
			sevenseg2 : out std_logic_vector(6 downto 0);
			errorled : out std_logic
		);
	end component;

	signal	wire_channel_sel :  headervaltype;
	signal	hex_wire0 :  std_logic_vector(3 downto 0);
	signal  ground : std_logic_vector(3 downto 0);

begin 

	inst0 : corr_filter_userio
	port map(channel_sel => user_chan_sel,
			channel_sel_io => hex_wire0,
			channel_sel_out => wire_channel_sel);
	
	inst1 : corr_filter
	generic map(head_size => work.corr_filter_pkg.header_size,
					out_bits => 11
				)
	port map(clk => clock,
			 rst => reset,
			 Yn_out => filter_out,
			 Xn_in => filter_in,
			 channel_sel => wire_channel_sel);
	
	
	inst2 : hex_counter
	port map(bits => hex_wire0,
			 bits2 => ground,
			 sevenseg => hex1_out,
			 sevenseg2 => hex2_out,
			 errorled => error_hex);
			 
	ground <= "0000";
	
end mappinz;